#include <msp/gl/matrix.h>
#include <msp/gl/renderer.h>
#include "libr2c2/tracktype.h"
#include "layout.h"
#include "path.h"
#include "track.h"
#include "tracktype.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Path3D::Path3D(const Track3D &t):
	Utility3D(t.get_layout()),
	track(t),
	path(t.get_track().get_active_path()),
	side(0),
	automatic(true),
	mesh(0),
	z_offs(0)
{
	update_mesh();

	layout.get_path_scene().add(*this);
	if(track.get_track().get_type().is_turnout())
		track.get_track().signal_path_changed.connect(sigc::mem_fun(this, &Path3D::path_changed));
}

Path3D::~Path3D()
{
	layout.get_path_scene().remove(*this);
}

void Path3D::set_automatic()
{
	automatic = true;
}

void Path3D::set_path(int p)
{
	if(p>=0 && !(track.get_track().get_type().get_paths()&(1<<p)))
		throw invalid_argument("Path3D::set_path");
	automatic = false;
	path = p;
	update_mesh();
}

void Path3D::set_side(int s)
{
	side = (s<0 ? -1 : s>0 ? 1 : 0);
	update_mesh();
}

void Path3D::set_color(const GL::Color &c)
{
	color = c;
}

void Path3D::set_layer(float l)
{
	z_offs = l*track.get_track().get_type().get_gauge()*0.01;
}

void Path3D::path_changed(unsigned p)
{
	if(automatic)
	{
		path = p;
		update_mesh();
	}
}

void Path3D::update_mesh()
{
	mesh = &track.get_type().get_path_mesh(path, side);
}

long Path3D::get_instance_key() const
{
	return reinterpret_cast<long>(&track.get_type());
}

void Path3D::render(GL::Renderer &renderer, const GL::Tag &tag) const
{
	if(tag=="unlit")
	{
		GL::MatrixStack::Push push_mtx(renderer.matrix_stack());
		renderer.matrix_stack() *= track.Object3D::get_matrix();
		renderer.matrix_stack() *= GL::Matrix::translation(0, 0, z_offs);

		glColor4f(color.r, color.g, color.b, color.a);
		mesh->draw(renderer);
	}
}

} // namespace R2C2
