#ifndef LIBR2C2_BLOCKALLOCATOR_H_
#define LIBR2C2_BLOCKALLOCATOR_H_

#include <list>
#include <msp/datafile/objectloader.h>
#include "blockiter.h"
#include "sensor.h"

namespace R2C2 {

class Train;

class BlockAllocator
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<BlockAllocator>
	{
	private:
		Block *prev_block;
		bool valid;

	public:
		Loader(BlockAllocator &);

	private:
		void block(unsigned);
		void hint(unsigned);
	};

	sigc::signal<void, Block &, Sensor *> signal_advanced;
	sigc::signal<void, Block &> signal_rear_advanced;

private:
	struct BlockMatch;

	typedef std::list<BlockIter> BlockList;

	Train &train;
	bool active;
	BlockList blocks;
	BlockList::iterator cur_blocks_end;
	Sensor *next_sensor;
	Block *pending_block;
	const Block *stop_at_block;
	bool reserving;
	bool advancing;

public:
	BlockAllocator(Train &);

	void set_active(bool);
	bool is_active() const { return active; }

	bool start_from(const BlockIter &);
	void rewind_to(const Block &);
	void clear();
	bool empty() const { return blocks.empty(); }
	void stop_at(const Block *);

	const BlockIter &first() const;
	const BlockIter &last() const;
	const BlockIter &last_current() const;
	const BlockIter &iter_for(const Block &) const;

	bool has_block(const Block &) const;
	bool is_block_current(const Block &) const;

private:
	void reserve_more();
	bool reserve_block(const BlockIter &);
	void advance_front(const Block *, bool);
	void advance_front(const Sensor *);
	void advance_back();
	void release_blocks_begin(const BlockList::iterator &);
	void release_blocks_end(const BlockList::iterator &);
	void release_block(const BlockList::iterator &);
public:
	void reverse();

private:
	void turnout_path_changing(Track &);
	void turnout_path_changed(Track &);
	void block_reserved(Block &, const Train *);
	void sensor_state_changed(Sensor &, Sensor::State);
	void update_next_sensor(Sensor *);
	void halt_event(bool);

public:
	void save(std::list<Msp::DataFile::Statement> &) const;
};

} // namepsace R2C2

#endif
